"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const baseSourceAdapter_1 = require("./baseSourceAdapter");
const path_1 = require("path");
const registry_1 = require("../../utils/registry");
const errors_1 = require("../../errors");
const fs_1 = require("fs");
const fileSystemHandler_1 = require("../../utils/fileSystemHandler");
const path_2 = require("../../utils/path");
/**
 * Handles types with mixed content. Mixed content means there are one or more additional
 * file(s) associated with a component with any file extension. Even an entire folder
 * can be considered "the content".
 *
 * __Example Types__:
 *
 * StaticResources, Documents, Bundle Types
 *
 * __Example Structures__:
 *
 *```text
 * foos/
 * ├── myFoo/
 * |   ├── fooFolder/
 * |      ├── foofighters.x
 * |   ├── foo.y
 * |   ├── fooBar.z
 * ├── myFoo.ext-meta.xml
 * bars/
 * ├── myBar.xyz
 * ├── myBar.ext2-meta.xml
 *```
 */
class MixedContentSourceAdapter extends baseSourceAdapter_1.BaseSourceAdapter {
    getRootMetadataXmlPath(trigger) {
        if (this.ownFolder) {
            // self contained components have all their files in their own folder
            const componentRoot = MixedContentSourceAdapter.trimPathToContent(trigger, this.type);
            return registry_1.findMetadataXml(componentRoot, path_1.basename(componentRoot));
        }
        return MixedContentSourceAdapter.findXmlFromContentPath(trigger, this.type);
    }
    populate(component, trigger) {
        let contentPath = MixedContentSourceAdapter.trimPathToContent(trigger, this.type);
        if (contentPath === component.xml) {
            contentPath = registry_1.findMetadataContent(path_1.dirname(contentPath), path_2.baseName(contentPath));
        }
        if (!fs_1.existsSync(contentPath)) {
            throw new errors_1.ExpectedSourceFilesError(this.type, trigger);
        }
        const ignore = new Set([component.xml]);
        const sources = fileSystemHandler_1.isDirectory(contentPath) ? fileSystemHandler_1.walk(contentPath, ignore) : [contentPath];
        component.sources = sources.filter(s => this.forceIgnore.accepts(s));
        return component;
    }
    /**
     * Trim a path up until the root of a component's content. If the content is a file,
     * the given path will be returned back. If the content is a folder, the path to that
     * folder will be returned. Intended to be used exclusively for MixedContent types.
     *
     * @param path Path to trim
     * @param type MetadataType to determine content for
     */
    static trimPathToContent(path, type) {
        const pathParts = path.split(path_1.sep);
        const typeFolderIndex = pathParts.findIndex(part => part === type.directoryName);
        const offset = type.inFolder ? 3 : 2;
        return pathParts.slice(0, typeFolderIndex + offset).join(path_1.sep);
    }
    /**
     * A utility for finding a component's root metadata xml from a path to a component's
     * content. "Content" can either be a single file or an entire directory. If the content
     * is a directory, the path can be files or other directories inside of it.
     *
     * @param path Path to content or a child of the content
     */
    static findXmlFromContentPath(path, type) {
        const rootContentPath = MixedContentSourceAdapter.trimPathToContent(path, type);
        const rootTypeDirectory = path_1.dirname(rootContentPath);
        const contentFullName = path_2.baseName(rootContentPath);
        return registry_1.findMetadataXml(rootTypeDirectory, contentFullName);
    }
}
exports.MixedContentSourceAdapter = MixedContentSourceAdapter;
//# sourceMappingURL=mixedContentSourceAdapter.js.map